// -*- mode: c++; mode: fold -*-
// Description								/*{{{*/
// $Id: rollback.h 97 2006-11-27 19:52:44Z jpra $
/* ######################################################################
 
   Rollback - TODO: Summary
 
   TODO: Description

   AUTHOR: David Salvador Pinheiro <david.pinheiro@caixamagica.pt>

   ##################################################################### */
									/*}}}*/
#ifndef PKGLIB_ROLLBACK_H
#define PKGLIB_ROLLBACK_H

#ifdef __GNUG__
#pragma interface "apt-pkg/rollback.h"
#endif

#include <apt-pkg/pkgcache.h>
#include <apt-pkg/depcache.h>

#include <vector>
#include <list>

//  JPRA: Commenting sqlite3 support. We only need a typedef here.
//  #include <sqlite3.h>

extern "C" {

typedef struct sqlite3 sqlite3;

}

using namespace std;

class pkgRollbackManager: protected pkgCache::Namespace
{
   protected:

   friend class pkgPackageManager;

   public:

   struct Item;
   class pkgCfgBackup;
   pkgRollbackManager(pkgDepCache *Cache);
   //AG: another useful construtor
   pkgRollbackManager();
   inline void SetCfgBackup( pkgCfgBackup* CfgBck ) {this->CfgBck = CfgBck;}
   inline pkgCfgBackup * GetCfgBackup() {return this->CfgBck;}
   virtual ~pkgRollbackManager();
   bool Commit();
   int GetTID();

   private:

   bool Debug;

   public:

   struct Item
   {
      enum Ops {Install, Remove, Purge, Upgrade, Downgrade} Op;

      string File;
      string Pkg;
      string Ver1;
      string Ver2;

      unsigned int TimeStamp;

      Item(Ops Op,string Pkg,string Ver1,string Ver2,unsigned int TimeStamp, string File = "") : Op(Op), File(File), Pkg(Pkg),Ver1(Ver1),Ver2(Ver2), TimeStamp(TimeStamp) {}
      Item() {}
   };

   protected:

   vector<Item> List;
   pkgDepCache *Cache;
   pkgCfgBackup *CfgBck;
   sqlite3 *hDb;
   int TIDCache;

   bool OpenDB();
   void CloseDB();
   bool CreateDBTables();
   bool DBTablesExist();
   bool InsertDB(const string &pkgName,const string &pkgVer1,const string &pkgVer2, const int Parent, const char Type, unsigned int TimeStamp);
   bool LogInstall(PkgIterator Pkg,string File);
   int GenerateTransactionID();
   bool LogPkgRemove(PkgIterator Pkg,bool Purge);
   bool LogPkgInstall(PkgIterator Pkg,string File);
   bool GetRollbackList(int TID,vector<Item> &List);
   bool GetRollbackTID(list<int> &tidList);

   public:

   bool GetRollbackList(vector<vector<Item> > &List);
   bool Rollback(int TID);
   bool RestoreCfgBackup();
};

class pkgRollbackManager::pkgCfgBackup
{
   public:

   bool Create(const char *pkgName,int TID);
   bool Restore();
   bool DeleteBckUPFiles();
   static string FormatFN(const char *pkgName,int TID, const char *pkgVersion, const char *pkgRelease);
   static string FormatFN(const char *pkgName,int TID, const char *pkgVersion);

   protected:

   virtual bool GetCfgFileList(const char *pkgName, vector<string> &FileList, string &pkgVersion, string &pkgRelease) {return false;}
   bool MarkRestore( const char *pkgName, const char *pkgVer,int TID);
   list<string> BckFileList;
   list<string> PkgCfgRestoreList;

   private:

   friend class pkgRollbackManager;
};

#endif


// vim:sts=3:sw=3
